"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from gr4vy.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated, TypedDict


class AccountUpdaterInquirySummaryTypedDict(TypedDict):
    id: str
    r"""The ID for the account updater inquiry."""
    payment_method_id: str
    r"""The ID of the payment method"""
    type: Literal["account-updater-inquiry"]
    r"""Always `account-updater-inquiry`"""


class AccountUpdaterInquirySummary(BaseModel):
    id: str
    r"""The ID for the account updater inquiry."""

    payment_method_id: str
    r"""The ID of the payment method"""

    TYPE: Annotated[
        Annotated[
            Optional[Literal["account-updater-inquiry"]],
            AfterValidator(validate_const("account-updater-inquiry")),
        ],
        pydantic.Field(alias="type"),
    ] = "account-updater-inquiry"
    r"""Always `account-updater-inquiry`"""
