"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


ProductType = Union[
    Literal[
        "physical",
        "discount",
        "shipping_fee",
        "sales_tax",
        "digital",
        "gift_card",
        "store_credit",
        "surcharge",
    ],
    UnrecognizedStr,
]
