"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


TransactionStatus = Union[
    Literal[
        "processing",
        "authorization_succeeded",
        "authorization_declined",
        "authorization_failed",
        "authorization_voided",
        "authorization_void_pending",
        "capture_succeeded",
        "capture_pending",
        "buyer_approval_pending",
    ],
    UnrecognizedStr,
]
