"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class GiftCardTokenTransactionCreateTypedDict(TypedDict):
    id: str
    r"""The ID for the gift card to charge."""
    amount: int
    r"""The monetary amount for this transaction to charge against the gift card, in the smallest currency unit for the given currency, for example `1299` cents to create an authorization for `$12.99`."""


class GiftCardTokenTransactionCreate(BaseModel):
    id: str
    r"""The ID for the gift card to charge."""

    amount: int
    r"""The monetary amount for this transaction to charge against the gift card, in the smallest currency unit for the given currency, for example `1299` cents to create an authorization for `$12.99`."""
